'===============================================================================
' GUI.bi
' Simple GUI for the FB graphics window, color depth 32 bit, char size 8x16
' First release Jan 17, 2015
' Latest release November 28, 2016
'===============================================================================

'Note:
'"GetMouse" is generally used without "Sleep 1" in order to enable a fast GUI, if
'many controls are used. It is recommended therefore to put "Sleep 1" once into 
'the event loop of the main program.


'------------------------------------------------------------------------------
' Types and Constants
'------------------------------------------------------------------------------

'Color constants:
Const WindowColor = &HEEEEEE, TextColor = &H000000, CursorColor = &HFF5555, _
			BorderColor = &HAAAAAA, TextBoxColor = &HFFFFFF, _
			HighlightBackColor = &H00AAEE, HighlightTextColor = &HFFFFFF, _
			ButtonColor = &HDDDDDD, ButtonClickedColor = &HCCCCCC, PanelFrameColor = &H0055AA

'Buffersize for ListBox:
Const LB_BufferSize = 1000

Type Button
  x As Integer		'Position left top
	y As Integer
	w As Integer		'Height
	h As Integer		'Width
	text As String	'label
End Type

Type CheckBox
  x As Integer
	y As Integer
	w As Integer
  h As Integer
	text As String
	state As Integer	'Check state (1 or 0)
End Type

Type RadioButton
  x As Integer
	y As Integer
	w As Integer
  h As Integer
	text As String
	state As Integer	'Check state (1 or 0)
End Type	

Type Label
  x As Integer		'Position left top
	y As Integer
	w As Integer		'Height
	h As Integer		'Width
	text As String	'label
End Type

Type TextBox
  x As Integer		'Position left top
	y As Integer
	w As Integer		'Height
	h As Integer		'Width
	text As String	'Text
  ReadOnly As Integer '0: editable, 1: not editable
  Style As Integer  '0: String, 1: Numeric, 2: Password
	CPos As Integer	'Cursor position
End Type

Type ListBox
  x As Integer					'Position left top
	y As Integer
	w As Integer					'Height
	h As Integer					'Width
	buffer(LB_BufferSize) As String	'Buffer for items
	imax As Integer				'Maximum index of items stored in the buffer
	offset As Integer			'Index of first item in the listbox when items are scrolled
	nmax as Integer				'maximum number of items visible in the listbox
	index As Integer			'Selected index
End Type

Type DataGrid
  x As Integer								'Position left top
	y As Integer
	w As Integer								'Height
	h As Integer								'Width
	m As Integer								'Row number 0 to m
	n As Integer								'Column Number 0 To n
	colwidth(100) As Integer		'Buffer for columnwidths
	colpos(100)	As Integer			'Left position of each column 
	buffer(100, 100) As String	'Buffer for items
	index_row As Integer				'Index of selected row
	index_col As Integer				'Index of selected column
	CPos As Integer							'Cursor position within a selected cell
End Type

Type TrackBar
  x As Integer					'Position left top
	y As Integer
	w As Integer					'Height
	h As Integer					'Width
	Style As Integer			'0 = horizontal, 1 = vertical
	minvalue As Integer		'Minimum and maximum Value
	maxvalue As Integer
	value As Integer			'Actual value
End Type

Type ProgressBar
  x As Integer					'Position left top
	y As Integer
	w As Integer					'Height
	h As Integer					'Width
	minvalue As Integer		'Minimum and maximum Value
	maxvalue As Integer
	value As Integer			'Actual value
End Type

Type Panel
  x As Integer
  y As Integer
  w As Integer
  h As Integer
  title As String
End Type

Type Frame
  x As Integer
  y As Integer
  w As Integer
  h As Integer
  title As String
End Type


'------------------------------------------------------------------------------
' Window
'------------------------------------------------------------------------------

Sub OpenWindow(ByVal w As Integer, ByVal h As Integer, ByVal title As String)
  'Window, textcolor black, backcolor white, 2 pages, char size 8x16

  Screenres w, h, 32, 2
  Windowtitle title
  Width w \ 8, h \ 16
  Color TextColor, WindowColor 
  Cls 0

End Sub


Function Window_Event_Close() As Integer
  'Returns 1 if the "Close" button ("x") of the window was clicked

	If Inkey = Chr(255, 107) Then Return 1 Else Return 0
	
End Function


'------------------------------------------------------------------------------
' Button
'------------------------------------------------------------------------------

Private Sub Button_Draw(ByVal btn As Button, ByVal colour As UInteger)
  'Draw a button - used by Subs "Button_New" and "Button_Event"

	Dim As Integer TextW, TextH
	
	Line (btn.x, btn.y) - (btn.x + btn.w, btn.y + btn.h), BorderColor, B
	Line (btn.x + 1, btn.y + 1) - (btn.x + btn.w - 1, btn.y + btn.h - 1), colour, BF 
	TextW = 8 * Len(btn.text)
	TextH = 16
	Draw String (btn.x + 0.5 * (btn.w - TextW), btn.y + 0.5 * (btn.h - TextH)), _
								btn.text, TextColor

End Sub
	

Function Button_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, _
										ByVal Text As String) As Button
  'Defines and draws a new button

	Dim As Button btn
	
	btn.x = x
  btn.y = y
  btn.h = h
  btn.w = w
  btn.text = text
		
	Button_Draw(btn, ButtonColor) 
	
  Return btn
	
End Function


Function Button_Event(ByVal btn As Button) As Integer
  'Returns 1 when button was clicked

	Dim As Integer mx, my, mbtn
  Dim As UInteger colour
    
  Getmouse(mx, my,, mbtn)
    
	If (mx >= btn.x) And (mx <= btn.x + btn.w) And (my >= btn.y) And (my <= btn.y + btn.h) _
		And mbtn = 1 Then
		'Active state of button:
		Button_Draw(btn, ButtonClickedColor)	
		'Wait until mouse button is released:
		Do
			Getmouse(mx, my,, mbtn)
			Sleep 1
		Loop Until mbtn = 0
		'Inactive state of button:
		Button_Draw(btn, ButtonColor)	
		Return 1
	Else
		Return 0
	End if
	
End Function


'------------------------------------------------------------------------------
' SetText
'------------------------------------------------------------------------------

Private Sub SetText(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, ByVal text As String, _
						ByVal Style As Integer, ByVal fgcolor As UInteger, ByVal bkcolor As UInteger)
  'Puts a text into a rectanglular area. If the text is too long, it will be 
  'truncated. Style: se Sub TextBox_New. This Sub is used by the following controls.
	
  Dim As Integer i
  
  Select Case Style 
  Case 0
    'String:
    Line (x + 1, y + 2) - (x + w - 1, y + h - 2), bkcolor, BF
    Draw String (x + 4, y + 0.5 * (h - 16)), Left(text, (w - 8) \ 8), fgcolor
  Case 1
    'Numeric: Text will only be dislayed if it is a number:
    For i = 1 To Len(text)
      If Asc(Mid(text, i, 1)) < 48 Or Asc(Mid(text, i, 1)) > 57 Then Exit Sub
    Next
    Line (x + 1, y + 2) - (x + w - 1, y + h - 2), bkcolor, BF
    Draw String (x + 4, y + 0.5 * (h - 16)), Left(text, (w - 8) \ 8), fgcolor
  Case 2
    'Password:
    Line (x + 1, y + 2) - (x + w - 1, y + h - 2), bkcolor, BF
    Draw String (x + 4, y + 0.5 * (h - 16)), String(Len(Left(text, (w - 8) \ 8)), "*"), fgcolor
  End Select  
    
End Sub


'------------------------------------------------------------------------------
' CheckBox
'------------------------------------------------------------------------------

Sub CheckBox_SetCheck(ByRef cb As CheckBox, ByVal State As Integer)
  'Set the check state of a checkbox to 0 or 1 and mark text of the Checkbox accordingly
	
  cb.State = State
	
	Line (cb.x + 4, cb.y + 0.5 * (cb.h - 16)) - (cb.x + 20, cb.y + 0.5 * cb.h + 8), WindowColor, BF
	Line (cb.x + 4, cb.y + 0.5 * (cb.h - 16)) - (cb.x + 20, cb.y + 0.5 * cb.h + 8), TextColor, B
	If cb.state = 1 Then Draw String (cb.x + 8, cb.y + 0.5 * (cb.h - 16)), "x", TextColor
	SetText(cb.x + 24, cb.y, cb.w - 24, cb.h, cb.text, 0, TextColor, WindowColor)
	
End Sub


Function CheckBox_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, _
  ByVal Text As String) As CheckBox
	'Define and draw a new CheckBox
  
	Dim As CheckBox cb
    
  cb.x = x
  cb.y = y
  cb.w = w
  cb.h = h
  cb.text = Text
  CheckBox_SetCheck(cb, 0)
  
  Return cb
	
End Function


Function CheckBox_Event(ByRef cb As CheckBox) As Integer
  'Returns 1 if checkbox was clicked and toggles the check state
	
	Dim As Integer mx, my, mb, State
      
  Getmouse(mx, my,, mb)
    
  If mx > cb.x And mx < (cb.x + cb.w) And my > cb.y And my < (cb.y + cb.h) And mb = 1 Then
		State = cb.State XOR 1
		CheckBox_SetCheck(cb, State)
		'Wait until mouse button is released:
		Do
			Getmouse(mx, my,, mb)
			Sleep 1
		Loop Until mb = 0
		Return 1
	Else
		Return 0
	End If
	
End Function


Function CheckBox_GetCheck(ByVal cb As CheckBox) As Integer
	'Returns the check state of a checkbox
	
	Return cb.State
	
End Function


'------------------------------------------------------------------------------
' RadioButton
'------------------------------------------------------------------------------

Sub RadioButton_SetCheck(ByRef rb As RadioButton, ByVal State As Integer)
  'Set the check state of a radio button to 0 or 1. Within an array of radiobuttons 
  '(see Function Radio_Button_Event), at most one radiobutton's check state shall
  'be set to one!
  
	rb.state = State
	
	Circle (rb.x + 12, rb.y + 0.5 * rb.h), 8, WindowColor,,,,F
	Circle (rb.x + 12, rb.y + 0.5 * rb.h), 8, TextColor
	If rb.state = 1 Then Circle (rb.x + 12, rb.y + 0.5 * rb.h), 3, TextColor,,,,F
	SetText(rb.x + 24, rb.y, rb.w - 24, rb.h, rb.text, 0, TextColor, WindowColor)
	
End Sub


Function RadioButton_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, _
  ByVal Text As String) As RadioButton
	'Define and draw a new radiobutton 

	Dim As RadioButton rb
  
  rb.x = x
  rb.y = y
  rb.w = w
  rb.h = h
  rb.text = Text
  RadioButton_SetCheck(rb, 0)
	
	Return rb
	
End Function


Function RadioButton_Event(rb() As RadioButton, Byref k As Integer) As Integer
  'Returns 1 if a radiobutton within an array of radiobuttons was clicked, 
  'otherwiese 0. k is the index of the radiobutton with check state 1 after a
  'radiobutton has been clicked.
  
	Dim As Integer n = UBound(rb), i, mx, my, mb, State, result
  
  k = -1    
  Getmouse(mx, my,, mb)
    
  For i = 0 To n
    If mx > rb(i).x And mx < (rb(i).x + rb(i).w) And my > rb(i).y _
      And my < (rb(i).y + rb(i).h) And mb = 1 Then
			'Set state of the clicked radiobutton to 1:
			If rb(i).state = 0 Then RadioButton_SetCheck(rb(i), 1)
			'Wait until mouse button is released:
			Do
				Getmouse(mx, my,, mb)
				Sleep 1
			Loop Until mb = 0
			k = i
			result = 1
			Exit For
		End If
	Next
	
	'Switch state of all other radiobuttons to 0:
	If k >= 0 Then
		For i = 0 To n
			If i <> k And rb(i).state = 1 Then RadioButton_SetCheck(rb(i), 0)
		Next
	End If
	
	Return result
	
End Function


'------------------------------------------------------------------------------
' Label
'------------------------------------------------------------------------------

Function Label_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, _
										ByVal text As String) As Label
  'Define and draw a new label

	Dim As Label lbl 
	
	SetText(x, y, w, h, text, 0, TextColor, WindowColor)

	lbl.x = x
  lbl.y = y
  lbl.h = h
  lbl.w = w
  lbl.text = text
		
  Return lbl
  
 End Function


'------------------------------------------------------------------------------
' EditText
'------------------------------------------------------------------------------

Private Sub EditText(ByVal x As Integer, ByVal y As Integer,ByVal w As Integer, ByVal h As Integer, _
						ByRef text As String, ByRef CPos As Integer, ByVal ReadOnly As Integer, _
            ByVal Style As Integer)
  'Edits the text within a rectangular aerea. 
  'CPos = Initial cursor position. 
  'ReadOnly, Style: See Function TextBox_New.
  'The Sub is used by TextBox and DataGrid.

	Dim As String  strKey
	Dim As Integer ascKey, Offset, MaxLen, mx, my, mb
	  
  MaxLen = w \ 8
    
  Do
    
    'Display text with cursor:
    ScreenLock
    SetText(x, y, w, h, Mid(text, Offset + 1, MaxLen), Style, TextColor, TextBoxColor)
    Draw String (x + 4 + 8 * (CPos - Offset), y + 0.5 * (h - 16)), "_", CursorColor
    ScreenUnlock
    
    'Request mouse state and set the cursor position:
    GetMouse mx,my,,mb
		Sleep 1 
		If mb = 1 And  mx > x And mx < (x + w) And my > y And my < y + h Then
			CPos = (mx - x - 4) \ 8 + Offset
			If CPos > Len(text) Then CPos = Len(text)
		End If
    
    'Return key:        
    strKey = Inkey
		ascKey = Asc(strKey)

		If strKey <> "" Then
			ascKey = Asc(strKey)
			
			If ascKey = 255 Then
			
				ascKey = Asc(Right(strKey, 1))
				Select Case ascKey
					Case 75
						'Move cursor left:
						If CPos > 0 Then
							CPos = CPos - 1
							If CPos < Offset Then Offset = Offset - 1
						End If
					Case 77	
						'Move cursor right:
						If CPos < Len(text) Then
							CPos = CPos + 1
							If CPos > MaxLen - 1 + Offset Then Offset = Offset + 1
						End If
					Case 71
						'Move Cursor to the beginning 
						CPos = 0
						Offset = 0
					Case 79
						'Move Cursor to the end
						CPos = Len(text)
						If Len(Text) > MaxLen Then Offset = Len(text) - MaxLen
					Case 83
						If ReadOnly = 0 Then  
							'Del
							If (Len(text) > 0) And (CPos < Len(text)) Then _
								text = Left(text, CPos) + Right(text, Len(text) - CPos - 1)
						End If
				End Select
				
			Else
			
				Select Case ascKey	
					Case 8
						If ReadOnly = 0 Then
						  'Backspace:
						  If (Len(text) > 0) And (CPos > 0) Then
								text = Left(text, CPos - 1) + Right(text, (Len(text) - CPos))
								CPos = CPos - 1
								If Offset > 0 Then Offset = Offset - 1
							End If
						End If
					Case 32 To 255
            If ReadOnly = 0 Then
              'Printable characters:
              Select Case Style
              Case 0, 2
                'String or Password:
                text = Left(text, CPos) + Chr(asckey) + Right(text, Len(text) - CPos)
                CPos = CPos + 1
              Case 1
                'Numeric:
                If asckey >= 48 And asckey <= 57 Then
                  text = Left(text, CPos) + Chr(asckey) + Right(text, Len(text) - CPos)
                  CPos = CPos + 1
                End If
              End Select  
              If CPos > MaxLen - 1 + Offset Then Offset = Offset + 1
            End If
        End Select  
				 
			End If
					
		End If
		
	Loop Until ascKey = 13 _
		Or (mb = 1 And (mx < x Or mx > (x + w) Or my < y Or my > (y + h)))	
					
	'End:
	Screenlock
	SetText(x, y, w, h, text, Style, TextColor, TextBoxColor)
	Screenunlock
	
End Sub				


'------------------------------------------------------------------------------
' TextBox
'------------------------------------------------------------------------------

Function TextBox_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, _
											ByVal text As String, ByVal ReadOnly As Integer = 0, ByVal Style As Integer = 0) _
                      As TextBox
  'Define and draw a new textbox.
  'ReadOnly = 0: text can be edited
  'ReadOnly = 1: text cannot be edited, only be scrolled (using TextBox_Edit)
  'Style = 0: String 
  'Style = 1: Numeric (only digits 0 to 9 are allowed)
  'Style = 2: Password (only asterisks are shown)

	Dim As TextBox tb
	
	Line (x, y) - (x + w, y + h), BorderColor, B
	SetText(x, y, w, h, text, Style, TextColor, TextBoxColor) 

	tb.x = x
  tb.y = y
  tb.h = h
  tb.w = w
  tb.text = text
  tb.ReadOnly = ReadOnly
  tb.Style = Style
  		
  Return tb
  
End Function


Sub TextBox_SetText(ByRef tb As TextBox, ByVal text As String)
'Set a text into a textbox

	SetText(tb.x, tb.y, tb.w, tb.h, text, tb.Style, TextColor, TextBoxColor)
	tb.text = text 

End Sub


Function TextBox_GetText(ByVal tb As Textbox) As String
'Get text from textbox

	Return tb.text
	
End Function


Sub TextBox_Edit(ByRef tb As Textbox)
'Edit text in textbox

	EditText(tb.x, tb.y, tb.w, tb.h, tb.text, tb.Cpos, tb.ReadOnly, tb.Style)

End Sub


Function TextBox_Event(ByRef tb As TextBox) As Integer
  'Returns 1 when textbox tb was clicked and stores the cursor position defined by
  'the mouseclick. 

	Dim As Integer mx, my, mbtn
    
  Getmouse(mx, my,, mbtn)
    
  If (mx >= tb.x) And (mx <= tb.x + tb.w) And (my >= tb.y) And (my <= tb.y + tb.h) _
		And mbtn = 1 Then
		tb.CPos = (mx - tb.x - 4) \ 8
		If tb.Cpos > Len(tb.text) Then tb.CPos = Len(tb.text)
  	Do
			Getmouse(mx, my,, mbtn)
			Sleep 1
		Loop Until mbtn = 0
		Return 1
	Else
		Return 0
	End If
	
End Function


'------------------------------------------------------------------------------
' ListBox
'------------------------------------------------------------------------------

Function ListBox_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer) As ListBox
'Draw and define new ListBox

	Dim As ListBox lb
	Dim As Integer i
	
	Line (x, y) - (x + w, y + h), BorderColor, B
	Line (x + 1, y + 1) - (x + w - 1, y + h - 1), TextBoxColor, BF
	
	'Draw a vertical sidebar at the right side with a symbol "up" (top) and
	'"down" (bottom:
	Line (x + w - 20, y) - (x + w - 20, y + h), BorderColor
	Line (x + w - 19, y + 1) - (x + w - 1, y + h - 1), ButtonColor, BF
	Line (x + w - 20, y + 20) - (x + w, y + 20), BorderColor
	Draw String (x + w - 12, y + 2), Chr(30), TextColor
	Line (x + w - 20, y + h - 20) - (x + w, y + h - 20), BorderColor
	Draw String (x + w - 12, y + h - 18), Chr(31), TextColor 
	
	lb.x = x
	lb.y = y
	lb.w = w
	lb.h = h
	For i = 0 To LB_BufferSize
		lb.buffer(i) = ""
	Next
	lb.imax = -1
	lb.offset = 0
	lb.nmax = h \ 16
	lb.index = -1
		
	Return lb
	
End Function


Sub DisplayItems(ByVal lb As ListBox)
'Display items in the listbox (used for vertical scrolling)
'This Sub is used by the ListBox.

	Dim As Integer i
	
	For i = 0 To lb.nmax - 1
		If lb.index > -1 And i + lb.offset = lb.index Then
			SetText(lb.x, lb.y + 16 * i, lb.w - 21, 16, lb.buffer(i + lb.offset), 0, _
							HighlightTextColor, HighlightBackColor)
		Else
			SetText(lb.x, lb.y + 16 * i, lb.w - 21, 16, lb.buffer(i + lb.offset), 0, _
              TextColor, TextBoxColor)
		End If
	Next
	
End Sub


Sub ListBox_Add(ByRef lb As ListBox, ByVal item As String)
'Add an item

	If lb.imax < LB_BufferSize Then
		lb.imax = lb.imax + 1
		lb.buffer(lb.imax) = item
		DisplayItems(lb)
	End If
	
End Sub
		

Function ListBox_GetMaxIndex(ByVal lb As ListBox) As Integer
'Returns the maximum index of items in the listbox

	Return lb.imax
	
End Function


Sub ListBox_SetItem(ByRef lb As ListBox, ByVal index As Integer, ByVal item As String)
'Set text of an existing item
	
	If index <= lb.imax Then
		DisplayItems(lb)
		lb.buffer(index) = item
	End If
	
End Sub


Sub ListBox_DeleteItem(ByRef lb As ListBox, ByVal index As Integer)
'Delete item at index

	Dim As Integer i
	
	If index <= lb.imax Then
		For i = index To lb.imax - 1
			lb.buffer(i) = lb.buffer(i + 1)
		Next
		lb.buffer(lb.imax) = ""
		lb.imax = lb.imax - 1
		DisplayItems(lb)
	End If
	
End Sub


Sub ListBox_InsertItem(ByRef lb As ListBox, ByVal item As String, ByVal index As Integer)
'Insert one item at index

	Dim As Integer i
	
	If lb.imax < LB_BufferSize And index <= lb.imax Then
		For i = lb.imax To index Step  -1
			lb.buffer(i + 1) = lb.buffer(i)
		Next
		lb.buffer(index) = item
		lb.imax = lb.imax + 1
		DisplayItems(lb)
	End If
	
End Sub


Sub ListBox_Clear(ByRef lb As ListBox)
'Delete all items in a textbox

	Dim As Integer i

	For i = 0 To lb.imax
		lb.buffer(i) = ""
	Next
	lb.index = -1
	lb.imax = -1
	lb.offset = 0
	DisplayItems(lb)
		
End Sub


Function ListBox_GetIndex(ByVal lb As ListBox) As Integer
'Returns the selected index 

	Return lb.index
	
End Function


Function ListBox_GetItem(ByVal lb As ListBox, ByVal index As Integer) As String
'Returns item text at index
 
	If index >= 0 Then Return lb.buffer(index) Else Return ""
		
End Function


Function ListBox_Event(ByRef lb As ListBox) As Integer
'Returns 1 when an item in the listbox was clicked. The selected index is stored
'as lb.index. If the symbol "up" or "down" is clicked, the items in the listbox 
'are scrolled vertical (also possible with mousewheel). 

	Dim As Integer i, index, mx, my, mb, event, mw0
	Static As Integer mw
	
	mw0 = mw      
	Getmouse(mx, my, mw, mb)
    
  event = 0
  
	If (mx > lb.x) And (mx < lb.x + lb.w - 20) And (my > lb.y) And (my < lb.y + 16 * lb.nmax) _
		And mb = 1 Then
		'Check if an item was clicked:
		i = (my - lb.y) \ 16 + lb.offset
		If i <= lb.imax Then
			lb.index = i
			event = 1
		Else
			lb.index = -1
		End If
		If event = 1 Then
			DisplayItems(lb)
			'Active state of item:
			SetText(lb.x, lb.y + 16 * (i - lb.offset), lb.w - 21, 16, lb.buffer(i), 0, _
							HighlightTextColor, HighlightBackColor) 
		End If
		'Wait until mousebutton is released:
		Do
			Getmouse(mx, my,,mb)
			Sleep 1
		Loop Until mb = 0
	End If
	
	'Scroll with mousewheel:
	If (mx > lb.x) And (mx < lb.x + lb.w - 20) And (my > lb.y) And (my < lb.y + lb.h) Then
		IF (mw - mw0 > 0) And lb.offset > 0 Then
			'Mousewheel up:
			lb.offset = lb.offset - 1
			DisplayItems(lb)
		ElseIF (mw - mw0) < 0 And (lb.imax - lb.offset) >= lb.nmax Then
			'Mousewheel down:
			lb.offset = lb.offset + 1
			DisplayItems(lb)
		End If
	End If
		
	'Scroll with symbol "up":
	If (mx > lb.x + lb.w - 20) And (mx < lb.x + lb.w) And (my > lb.y) _
		And (my < lb.y + 20) And mb = 1 Then
		Do
			Getmouse(mx, my,,mb)
			Sleep 20
			If (mx > lb.x + lb.w - 20) And (mx < lb.x + lb.w) And (my > lb.y) _
				And (my < lb.y + 20) And lb.offset > 0 Then
				lb.offset = lb.offset - 1
				DisplayItems(lb)
			End If
		Loop Until mb = 0  
	End If
	
	'Scroll with symbol "down":
	IF (mx > lb.x + lb.w - 20) And (mx < lb.x + lb.w) And (my > lb.y + lb.h - 20) _ 
		And (my < lb. y + lb.h) And mb = 1 Then
		Do
			Getmouse(mx, my,,mb)
			Sleep 20
			If (mx > lb.x + lb.w - 20) And (mx < lb.x + lb.w) And (my > lb.y + lb.h - 20) _ 
				And (my < lb. y + lb.h) And (lb.imax - lb.offset) >= lb.nmax Then
				lb.offset = lb.offset + 1
				DisplayItems(lb)
			End If
		Loop Until mb = 0
	End If	
		
	Return event
	
End Function


'------------------------------------------------------------------------------
' DataGrid
'------------------------------------------------------------------------------

Function DataGrid_New(ByVal x As Integer, ByVal y As Integer, ByVal m As Integer, ByVal n As Integer, _
											cw() As Integer) As DataGrid
'Draw and define new datagrid. 
'x, y: position left top
'm, n: maximum indexes of rows and columns
'cw(): columnwidths

	Dim As DataGrid dg
	Dim As Integer i, j, k = Ubound(cw)
	
	'Define all parameters:
	dg.x = x
	dg.y = y
	dg.m = m
	dg.n = n
	dg.colwidth(0) = cw(0)
	dg.colpos(0) = dg.x
	For j = 1 To k
		dg.colwidth(j) = cw(j)
		dg.colpos(j) = dg.colpos(j - 1) + dg.colwidth(j - 1)
	Next
	For i = 0 To 100
		For j = 0 To 100
			dg.buffer(i, j) = ""
		Next
	Next
	dg.index_row = -1
	dg.index_col = - 1
	dg.w = dg.colpos(k) + dg.colwidth(k) - dg.x
	dg.h = 20 * (m + 1)
	
	'Draw Data grid:
	Line (dg.x, dg.y) - (dg.x + dg.w, dg.y + dg.h), BorderColor, B
	Line (dg.x + 1, dg.y + 1) - (dg.x + dg.w - 1, dg.y + dg.h - 1), TextBoxColor, BF
	For i = 1 To m
		Line (dg.x, dg.y + i * 20) - (dg.x + dg.w, dg.y + i * 20), BorderColor
	Next
	For j = 1 To n
		Line (dg.colpos(j), dg.y) - (dg.colpos(j), dg.y + dg.h), BorderColor
	Next
	
	Return dg
			
End Function


Sub DataGrid_SetItem(ByRef dg As DataGrid, ByVal i As Integer, ByVal j As Integer, _
											ByVal item As String)
'Set an item into the data grid, row i, column j

	If i <= dg.m And j <= dg.n Then
		SetText(dg.colpos(j), dg.y + 20 * i, dg.colwidth(j), 20, item, 0, TextColor, TextBoxColor)
		dg.buffer(i, j) = item
	End If
		
End Sub


Sub DataGrid_Clear(ByRef dg As DataGrid)
'Delete all items of data grid

	Dim As Integer i, j

	For i = 0 To 100
		For j = 0 To 100
			DataGrid_SetItem(dg, i, j, "") 
		Next
	Next
		
End Sub


Sub DataGrid_GetIndexes(ByVal dg As DataGrid, ByRef i As Integer, ByRef j As Integer)
'Returns selected indexes of data grid

	i = dg.index_row
	j = dg.index_col
	
End Sub


Function DataGrid_GetItem(ByVal dg As DataGrid, ByVal i As Integer, ByVal j As Integer) As String
'Returns item at row i, column j

	Return dg.buffer(i, j)
	
End Function


Sub DataGrid_EditItem(ByRef dg As DataGrid, ByVal i As Integer, ByVal j As Integer, _
											ByVal ReadOnly As Integer = 0)
'Edit item at row i, column j

	If i <= dg.m And j <= dg.n Then _
		EditText(dg.colpos(j), dg.y + 20 * i, dg.colwidth(j), 20, dg.buffer(i, j),_
							dg.CPos, ReadOnly, 0)
			
End Sub


Function DataGrid_Event(ByRef dg As DataGrid) As Integer
'Returns 1 if data grid is clicked and stores indexes of selected item  

	Dim As Integer i, j, mx, my, mb, event
	Dim As String text
      
  Getmouse(mx, my,, mb)
    
	If (mx >= dg.x) And (mx <= dg.x + dg.w) And (my >= dg.y) And (my <= dg.y + dg.h) _
		And mb = 1 Then
		i = (my - dg.y) \ 20
		For j = 0 To dg.n
			If dg.colpos(j) > mx Then Exit For
		Next
		j = j - 1
		'Active state of item:
		SetText(dg.colpos(j), dg.y + 20 * i, dg.colwidth(j), 20, dg.buffer(i, j), 0, _
						HighlightTextColor, HighlightBackColor) 
		'Wait until mouse button is released:
		Do
			Getmouse(mx, my,, mb)
			Sleep 1
		Loop Until mb = 0
		'Inactive state of item :
		SetText(dg.colpos(j), dg.y + 20 * i, dg.colwidth(j), 20, dg.buffer(i, j), 0, _
						TextColor, TextBoxColor)
		dg.index_row = i
		dg.index_col = j
		dg.CPos = (mx - dg.colpos(j) - 4) \ 8
		If dg.CPos > Len(dg.buffer(i, j)) Then dg.CPos = Len(dg.buffer(i, j))
  	event = 1
	Else
		event = 0
	End if
	
	Return event
	
End Function


'------------------------------------------------------------------------------
' Trackbar
'------------------------------------------------------------------------------

'Private Sub DrawSlide(ByVal tb As TrackBar, ByVal Value As Integer)
''Draws the slide within a trackbar according to its value. This Sub is used by
''the trackbar. 
''Minimum slider position = tb.x + 5
''Maximum slider position: tb.x + tb.w - 5
'
'	Dim As Integer xpos
'	
'	xpos = tb.x + 5 + (tb.value - tb.minvalue) * (tb.w - 10) / (tb.maxvalue - tb.minvalue)
'	Line (tb.x + 1, tb.y + 1) - (tb.x + tb.w - 1, tb.y + tb.h - 1), TextBoxColor, BF  
'	Line (xpos - 4, tb.y + 1) - (xpos + 4, tb.y + tb.h - 1), BorderColor, B
'	Line (xpos - 3, tb.y + 2) - (xpos + 3, tb.y + tb.h - 2), ButtonColor, BF
'	
'End Sub


Private Sub DrawTrackBar(ByRef tb As TrackBar, ByVal Value As Integer)
  'Draws the trackbar according to its value. This Sub is used by the trackbar.
  'Style = 0 (horizontal trackbar):
  'Minimum slider position = tb.x + 5
  'Maximum slider position = tb.x + tb.w - 5
  'Style = 1 (vertical trackbar):
  'Minimum slider position = tb.y + h - 5
  'Maximum slider position = tb.y + 5 

	Dim As Integer xpos, ypos
	
	Select Case tb.Style
		Case 0
			'horizontal
			xpos = tb.x + 5 + (tb.Value - tb.minvalue) * (tb.w - 10) / (tb.maxvalue - tb.minvalue)
			Line (tb.x, tb.y) - (tb.x + tb.w, tb.y + tb.h), TextBoxColor, BF
			Line (tb.x, tb.y) - (tb.x + tb.w, tb.y + tb.h), BorderColor, B
			Line (xpos - 3, tb.y + 2) - (xpos + 3, tb.y + tb.h - 2), ButtonColor, BF  
			Line (xpos - 3, tb.y + 2) - (xpos + 3, tb.y + tb.h - 2), BorderColor, B
		Case 1
			'vertikal
			ypos = tb.y + tb.h - 5 - (tb.Value - tb.minvalue) * (tb.h - 10) / (tb.maxvalue - tb.minvalue)
			Line (tb.x, tb.y) - (tb.x + tb.w, tb.y + tb.h), TextBoxColor, BF
			Line (tb.x, tb.y) - (tb.x + tb.w, tb.y + tb.h), BorderColor, B
			Line (tb.x + 2, ypos - 3) - (tb.x + tb.w - 2, ypos + 3), ButtonColor, BF
			Line (tb.x + 2, ypos - 3) - (tb.x + tb.w - 2, ypos + 3), BorderColor, B
		End Select			
		
End Sub


Function TrackBar_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, _
	ByVal h As Integer, ByVal Style As Integer = 0) As Trackbar
	'Draw and define a new Trackbar.
	'Style = 0: horizontal, Sytyle = 1: vertical  
	
	Dim As TrackBar tb
	
	Line (x, y) - (x + w, y + h), BorderColor, B
	Line (x + 1, y + 1) - (x + w - 1, y + h - 1), TextBoxColor, BF
	
	tb.x = x
	tb.y = y 
	tb.w = w
	tb.h = h
	tb.Style = Style
	tb.minvalue = 0
	tb.maxvalue = 100
	tb.value = tb.minvalue
	
	DrawTrackBar(tb, tb.Value)
	
	Return tb
	
End Function


Sub TrackBar_SetRange(ByRef tb As TrackBar, ByVal minvalue As Integer, ByVal maxvalue As Integer)
'Set the range of a trackbar

	tb.minvalue = minvalue
	tb.maxvalue = maxvalue
	
End Sub


Sub TrackBar_SetValue(ByRef tb As TrackBar, ByVal Value As Integer)
'Sets the value of a trackbar and draws the slide

	tb.value = value
	DrawTrackBar(tb, Value)
	
End Sub


Function TrackBar_Event(ByRef tb As TrackBar) As Integer
'Returns 1 when Trackbar was clicked, moves the slide to the clicked position
'and updates the value of the trackbar

'	Dim As Integer mx, my, mbtn, value
'      
'  Getmouse(mx, my,, mbtn)
'    
'  If (mx >= tb.x + 5) And (mx <= tb.x + tb.w - 5) And (my >= tb.y) And (my <= tb.y + tb.h) _
'		And mbtn = 1 Then
'		Screenlock
'		value = tb.minvalue + (mx - tb.x - 5) * (tb.maxvalue - tb.minvalue) / (tb.w - 10) 
'		TrackBar_SetValue(tb, value)
'		Screenunlock
'		Return 1
'	Else
'		Return 0
'	End if

	Dim As Integer mx, my, mbtn, Value
      
  Getmouse(mx, my,, mbtn)
  
  Select Case tb.Style
		
		Case 0
			'horizonal:
			If (mx >= tb.x + 5) And (mx <= tb.x + tb.w - 5) And (my >= tb.y) And (my <= tb.y + tb.h) _
				And mbtn = 1 Then
				Screenlock
				Value = tb.minvalue + (mx - tb.x - 5) * (tb.maxvalue - tb.minvalue) / (tb.w - 10) 
				TrackBar_SetValue(tb, Value)
				Screenunlock
				Return 1
			Else
				Return 0
			End If
			
		Case 1
			'vertikal:
			If (mx >= tb.x) And (mx <= tb.x + tb.w) And (my >= tb.y + 5) And (my <= tb.y + tb.h - 5) _
				And mbtn = 1 Then
				Screenlock
				Value = tb.minvalue - (my - (tb.y + tb.h - 5)) * (tb.maxvalue - tb.minvalue) / (tb.h - 10)
				TrackBar_SetValue(tb, Value)
				Screenunlock
				Return 1
			Else
				Return 0
			End If
		
		End Select
	
End Function


Function TrackBar_GetValue(ByRef tb As Trackbar) As Integer
'Returns the actual value of the trackbar

	Return tb.value
	
End Function


'------------------------------------------------------------------------------
' Progressbar
'------------------------------------------------------------------------------

Private Sub DrawBar(ByRef pb As ProgressBar, ByVal Value As Integer)
'Draws the bar within a progressbar according to its value. This Sub is used by
'the progressbar. 
'Minimum position = pb.x + 2
'Maximum position: pb.x + pb.w - 2

	Dim As Integer xpos
	
	xpos = pb.x + 2 + (pb.value - pb.minvalue) * (pb.w - 4) / (pb.maxvalue - pb.minvalue)
	Line (pb.x + 2, pb.y + 2) - (xpos, pb.y + pb.h - 2), HighlightBackColor, BF
	
End Sub


Function ProgressBar_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer) As ProgressBar
	'Draw and define a new horizontal progressbar. 
	
	Dim As ProgressBar pb
	
	Line (x, y) - (x + w, y + h), BorderColor, B
	Line (x + 1, y + 1) - (x + w - 1, y + h - 1), TextBoxColor, BF
	
	pb.x = x
	pb.y = y 
	pb.w = w
	pb.h = h
	pb.minvalue = 0
	pb.maxvalue = 100
	pb.value = pb.minvalue
	
	Return pb
	
End Function


Sub ProgressBar_SetRange(ByRef pb As ProgressBar, ByVal minvalue As Integer, ByVal maxvalue As Integer)
'Set the range of a ProgressBar

	pb.minvalue = minvalue
	pb.maxvalue = maxvalue
	
End Sub


Sub ProgressBar_SetValue(ByRef pb As ProgressBar, ByVal Value As Integer)
'Sets the value of a ProgressBar and draws the bar

	If value >= pb.minvalue And value <= pb.maxvalue Then
		pb.value = value
		DrawBar(pb, value)
	End If
	
End Sub


Sub ProgressBar_Clear(ByRef pb As ProgressBar)
'Deletes the content of the prgressbar

	Line (pb.x + 1, pb.y + 1) - (pb.x + pb.w - 1, pb.y + pb.h - 1), TextBoxColor, BF
	
End Sub
 

'------------------------------------------------------------------------------
' Panel
'------------------------------------------------------------------------------

Function Panel_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal h As Integer, ByVal title As String)_
  As Panel
  'Define and draw a new Panel
  
  Dim As Panel pnl
  
  'Frame and window:
  Line (x, y) - (x + w, y + h), PanelFrameColor, B
  Line (x + 1, y + 1) - (x + w - 1, y + h - 1), WindowColor, BF
  
  'Title, titlebar height = 20 pixels:
  Line (x, y) - (x + w, y + 20), PanelFrameColor, BF
  Draw String (x + 10, y + 2), title, HighLightTextColor
  
  pnl.x = x
  pnl.y = y
  pnl.w = w
  pnl.h = h
  pnl.title = title
  
  Return pnl
  
End Function


'-------------------------------------------------------------------------------
'Frame
'-------------------------------------------------------------------------------

Function Frame_New(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, _
										ByVal h As Integer, ByVal title As String) _
  As Frame
  'Neuen Rahmen zeichnen und definieren
  
  Dim As Frame frm
  
  Line (x, y) - (x + w, y + h), BorderColor, B
  SetText(x + 8, y - 8, 8 * (Len(title) + 1), 16, title, 0, TextColor, WindowColor)
  
  frm.x = x
  frm.y = y
  frm.w = w
  frm.h = h
  frm.title = title
  
  Return frm
  
End Function 
  

'------------------------------------------------------------------------------
' InputBox
'------------------------------------------------------------------------------
  
Function InputBox(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal Title As String, _
                  ByVal Prompt As String, ByRef Text As String = "", ByVal Style As Integer = 0) As String
  'Inputbox, height = 110 pixels, returns an edited text.
  'Prompt = e.g. question
  'Style = 0: String
  'Style = 1: Numeric
  'Style = 2: Password
  '(See Sub TextBox_New)
                  
  Dim As Panel pnl
  Dim As Label lb
	Dim As TextBox tb
	Dim  As Button btn
	Dim As Integer i
	
	'Save the screen and draw the inputbox:
	ScreenCopy 0, 1
	pnl = Panel_New   (x, y, w, 110, Title)
  lb = Label_New    (pnl.x + 10, pnl.y + 30, pnl.w - 20, 20, Prompt) 
  tb = TextBox_New  (pnl.x + 10, pnl.y + 50, pnl.w - 20, 20, Text, Style)
	btn = Button_New  (pnl.x + pnl.w / 2 - 30, y + 80, 60, 20, "Ok")
	
	'Event-Loop:
	Do
		If TextBox_Event(tb) Then TextBox_Edit(tb)
    Sleep 1
	Loop Until Button_Event(btn)
	
	'Restore the screen and return the text:
	ScreenCopy 1, 0
	Return TextBox_GetText(tb)
	
End Function


'------------------------------------------------------------------------------
' MessageBox
'------------------------------------------------------------------------------

Function MsgBox(ByVal x As Integer, ByVal y As Integer, ByVal w As Integer, ByVal Titel As String, ByVal Text As String, _
								ByVal ButtonText0 As String = "Ok", ByVal ButtonText1 As String = "", _
								ByVal ButtonText2 As String = "") As Integer
  'Messagebox, height = 20 pixels, up to three buttons number 0 to 20, arranged 
  'from left to right. The function returns the number of the clicked button.
  
  Dim As Panel pnl
  Dim As Label lbl
	Dim As Button btn0, btn1, btn2
	Dim As Integer i, w0, w1, w2, result
		
	'Save the screen and draw the messagebox:
	ScreenCopy 0, 1
  pnl = Panel_New(x, y, w, 90, Titel)
  lbl = Label_New(pnl.x + 10, pnl.y + 30, pnl.w - 20, 20, Text)
  	
	'Place the buttons:
  w0 = 8 * (Len(ButtonText0) + 2)
  btn0 = Button_New(pnl.x + 10, pnl.y + 60, w0, 20, ButtonText0)
  If Len(ButtonText1) > 0 Then
    w1 = 8 * (Len(ButtonText1) + 2)
    btn1.x = pnl.x + w0 + 20
    btn1 = Button_New(btn1.x, pnl.y + 60, w1, 20, ButtonText1)
  End If
  If Len(ButtonText2) > 0 Then
    w2 = 8 * (Len(ButtonText2) + 2)
    btn2.x = pnl.x + w0 + w1 + 40
    btn2 = Button_New(btn2.x, pnl.y + 60, w2, 20, ButtonText2)
  End If
    
  'Event-Loop:
	result = -1
	Do
		If Button_Event(btn0) Then result = 0
		If Button_Event(btn1) Then result = 1
		If Button_Event(btn2) Then result = 2
    Sleep 1
	Loop Until result >= 0
	
	'Restore the screen and return the number of the clicked button:
	ScreenCopy 1, 0
	Return result
	
End Function	


